#include "dht.h"
#include "LogEDht.h"
#include "RTClib.h"
#include "Fishino.h"
#include <SD.h>

extern RTC_DS1307 Rtc;
extern DateTime OraCorrente;
extern SdFile file;
extern SdFile root;

boolean MemoriaIngresso = 0;
int SommaTemperatura = 0;
unsigned char Campioni = 0;
unsigned int SommaUmidita = 0;
boolean ErroreDht = false;
boolean ErroreSd = false;
unsigned char StatoLog;

dht Dht;

void SetupLog()
{
  pinMode(LOG_LED,OUTPUT);
  digitalWrite(LOG_LED,LOW);
}

void LogS(unsigned char Stacco, unsigned char StatoTemp)
{
  static unsigned long UltimoLancio = 0;
  static unsigned char i = 0;
  static unsigned char ContaMedia = 0;
  if(millis() - UltimoLancio > TEMPO_BASE)
  {
    UltimoLancio = millis();
    i++;
    if(i > Stacco)
    {
      if( (i % 2) == 1)
        digitalWrite(LOG_LED,HIGH);
      else
        digitalWrite(LOG_LED,LOW);
      if( ( (i - Stacco) / 2 ) >= StatoLog)
      {
        if(StatoTemp != 0)
          StatoLog = StatoTemp;
        i = 0;
      }
    }
    if(ContaMedia++ == 0)
      MediaValori();
  }
}

void Log(unsigned char StatoTemp)
{
  LogS(STACCO, StatoTemp);
}

void LogLungo(unsigned char StatoTemp)
{
  LogS(STACCO_L, StatoTemp);
}

void LoopLog()
{
  if(ErroreDht)
    Log(LOG_DHT_ERROR);
  else
  {
    if(Rtc.isrunning())
    {
      if(ErroreSd)
        Log(LOG_SD_ERROR);
      else
      {
        switch(Fishino.status())
        {
          case STATION_CONNECT_FAIL: case STATION_TIMEOUT: case STATION_BAD_SSID:
            Log(LOG_CONNECT_ERROR);
            break;
          case STATION_NO_AP_FOUND:
            Log(LOG_NO_AP_FOUND);
            break;
          case STATION_WRONG_PASSWORD:
            Log(LOG_PASSWORD_ERROR);
            break;
          case STATION_GOT_IP:
            Log(LOG_OK);
            break;
          default:
            Log(LOG_NULL);
            break;
        }
      }
    }
    else
      Log(LOG_RTC_ERROR);
  }
}

void MediaValori()
{ // Entro qui ogni 25.5 secondi
  if(Dht.read(DHT22_PIN) == DHTLIB_OK)
  {
    ErroreDht = false;
    if(Rtc.isrunning())
    {
      if(Campioni++ < 60)
      {
        SommaTemperatura += getTemperatura();
        SommaUmidita += getUmidita();
        if(StatoIngresso)
          MemoriaIngresso = 1;
        if(ModoTermostato && StatoUscita)
          MemoriaIngresso = 1;
      }
    }
  }
  else
    ErroreDht = true;
}

void RegistraValori()
{
  static unsigned char UltimoMinuto = 255;
  const char Zero = '0';
  char* CarattereLetto;
  int RighePresenti, Temp;
  char NomeFile[13];
  NomeFile[0] = '2'; NomeFile[1] = Zero; NomeFile[2] = Zero; NomeFile[3] = Zero; NomeFile[4] = Zero; NomeFile[5] = Zero; NomeFile[6] = Zero; NomeFile[7] = Zero; NomeFile[8] = '.'; NomeFile[9] = 't'; NomeFile[10] = 'x'; NomeFile[11] = 't'; NomeFile[12] = '\0';
  if( (Campioni > 0) && Rtc.isrunning() && (!file.isOpen()) && ( (OraCorrente.minute() % 5) == 0) && (UltimoMinuto != OraCorrente.minute()) )
  {
    // Preparazione Nome File
    NomeFile[1] = (OraCorrente.year() - 2000) / 100;
    NomeFile[2] = (OraCorrente.year() - 2000 - (NomeFile[1] * 100) ) / 10;
    NomeFile[3] = (OraCorrente.year() - 2000 - (NomeFile[1] * 100) - (NomeFile[2] * 10) );
    NomeFile[4] = OraCorrente.month() / 10;
    NomeFile[5] = OraCorrente.month() - (NomeFile[4] * 10);
    NomeFile[6] = OraCorrente.day() / 10;
    NomeFile[7] = OraCorrente.day() - (NomeFile[6] * 10);
    for(RighePresenti=1;RighePresenti<8;RighePresenti++)  // Uso RighePresenti come variabile temporanea
      NomeFile[RighePresenti] += Zero;
    // Guardo se il file esiste e ci sono gia dei dati
    RighePresenti = 0;
    if(file.open(&root, NomeFile, O_READ))
    { // File Esiste: Leggo il contenuto
      while( (file.read(CarattereLetto,1) > 0) && (RighePresenti < DATI_PER_GIORNO) )
        if(*CarattereLetto == '\n')
          RighePresenti++;  // Conto le righe
      file.close();
    }
    // Apro in scrittura
    if(file.open(&root, NomeFile, O_CREAT | O_APPEND | O_WRITE )) // O_SYNC
    {
      ErroreSd = false;
      UltimoMinuto = OraCorrente.minute();
      if(RighePresenti <= getNumeroInGiorno(OraCorrente.hour(),OraCorrente.minute()))
      {
        while(RighePresenti++ < getNumeroInGiorno(OraCorrente.hour(),OraCorrente.minute()))
          file.write("null,null,null\n",15);
        // Scrittura Temperatura
        Temp = SommaTemperatura / Campioni;
        file.write((Temp / 100) + Zero);
        Temp = Temp - ((Temp / 100) * 100);
        file.write((Temp / 10) + Zero);
        Temp = Temp - ((Temp / 10) * 10);
        file.write(Temp + Zero);
        file.write(FILE_SEPARATOR);
        // Scrittura Umidita
        Temp = SommaUmidita / Campioni;
        file.write((Temp / 100) + Zero);
        Temp = Temp - ((Temp / 100) * 100);
        file.write((Temp / 10) + Zero);
        Temp = Temp - ((Temp / 10) * 10);
        file.write(Temp + Zero);
        file.write(FILE_SEPARATOR);
        // Scrittura Stato
        if(MemoriaIngresso)
          file.write(1 + Zero);
        else
          file.write(Zero);
        file.write('\n');
        Campioni = 0;
        SommaTemperatura = 0;
        SommaUmidita = 0;
        MemoriaIngresso = 0;
      }
      file.close();
    }
    else
      ErroreSd = true;
  }
}

