#include <SPI.h>
#include "Fishino.h"
#include "Flash.h"
#include <SD.h>
#include "FishinoWebServer.h"
#include "LogEDht.h"
#include "RTClib.h"

extern RTC_DS1307 Rtc;
extern DateTime OraCorrente;
extern dht Dht;
extern boolean StatoUscita;

const int SD_CS = 4;

boolean file_handler(FishinoWebServer& web_server);
boolean index_handler(FishinoWebServer& web_server);
void file_uploader_handler(FishinoWebServer& web_server, FishinoWebPutHandler::PutAction action, char* buffer, int size);

extern unsigned char StatoLog;
Sd2Card card;
SdVolume volume;
SdFile root;
SdFile file;

void send_file_name(FishinoWebServer& web_server, char* filename);

FishinoWebServer::PathHandler handlers[] =
{
  {"/", FishinoWebServer::GET, &index_handler },
  {"/" "*", FishinoWebServer::GET, &file_handler },
  {"/upload/" "*", FishinoWebServer::PUT, &FishinoWebPutHandler::put_handler },
  {NULL},
};

const char* headers[] =
{
  "Content-Length",
  NULL
};

void file_uploader_handler(FishinoWebServer& web_server, FishinoWebPutHandler::PutAction action, char* buffer, int size)
{
  static unsigned char Azione = 0;
  const char* fname = web_server.get_path();
  if(action == FishinoWebPutHandler::START)
  {
    Azione = 0;
    if(fname[8] == 'h' && fname[9] == '.')
      Azione = 1;
    if(fname[8] == 'o' && fname[9] == '.')
      Azione = 2;
  }
  else
  {
    if(Azione == 1)
      Rtc.adjust(DateTime(buffer,buffer + 11));
    if(Azione == 2)
      if(size > 0)
        if(buffer[0] == '1')
          AccendiUscita();
        else
          SpengiUscita();
    Azione = 0;
  }
}

void send_file_name(FishinoWebServer& web_server, char* filename)
{
  const char Accapo = '\n';
  if( (filename[0] == 'C') && (filename[1] == 'O') && (filename[2] == 'N') && (filename[3] == 'F') && (filename[4] == '.') && (filename[5] == 'T') && (filename[6] == 'X') && (filename[7] == 'T'))
  {
    web_server << OraCorrente.year();
    web_server << Accapo;
    web_server << OraCorrente.month();
    web_server << Accapo;
    web_server << OraCorrente.day();
    web_server << Accapo;
    web_server << OraCorrente.hour();
    web_server << Accapo;
    web_server << OraCorrente.minute();
    web_server << Accapo;
    web_server << StatoLog;
    web_server << Accapo;
    web_server << getTemperatura();
    web_server << Accapo;
    web_server << getUmidita();
    web_server << Accapo;
    if(StatoIngresso)
      web_server << '1';
    else
      web_server << '0';
    web_server << Accapo;
    if(StatoUscita)
      web_server << '1';
    else
      web_server << '0';
    web_server << Accapo;
    return;
  }
  FishinoWebServer::MimeType mime_type = FishinoWebServer::get_mime_type_from_filename(filename);
  if (file.open(&root, filename, O_READ))
  {
    web_server.send_error_code(200);
    web_server.send_content_type(mime_type);
    web_server.end_headers();
    web_server.send_file(file);
    file.close();
  }
  else
  {
    web_server.send_error_code(404);
    web_server.end_headers();
    web_server << F("404 - File non trovato") << filename << Accapo;
  }
}

boolean file_handler(FishinoWebServer& web_server)
{
  unsigned char i;
  char* filename = FishinoWebServer::get_file_from_path(web_server.get_path());
  // Non considero il GET
  for(i=0;filename[i] != '\0';i++)
    if(filename[i] == '?')
      filename[i] = '\0';
  if (!filename)
  {
    web_server.send_error_code(400);
    web_server << F("Bad Request");
    return true;
  }
  send_file_name(web_server, filename);
  free(filename);
  return true;
}

boolean index_handler(FishinoWebServer& web_server)
{
  send_file_name(web_server, "INDEX.HTM");
  return true;
}

unsigned char LeggiRiga(SdFile& file, char* Temp)
{
  unsigned char i = 0, j = 0;
  while(1)
  {
    if( (!(file.read(Temp + i,1) > 0)) || (Temp[i] == '\n') )
    {
      j = i;
      break;
    }
    if( (Temp[i] == '\r') || (Temp[i] == '\0') )
      j = i;
    i++;
  }
  Temp[j] = '\0';
  return j;
}

boolean has_ip_address = false;
FishinoWebServer web = FishinoWebServer(handlers, headers);

void SetupWeb()
{
  // Inizializzazione Fishino
  SPI.begin();
  SPI.setClockDivider(SPI_CLOCK_DIV2);
  while(!Fishino.reset());
  Fishino.setPhyMode(PHY_MODE_11B);
  Fishino.setMode(STATION_MODE);
  // Inizializzazione SD
  pinMode(SD_CS, OUTPUT);
  digitalWrite(SD_CS, HIGH);
  while ( (!card.init(SPI_FULL_SPEED, SD_CS)) || (!volume.init(&card)) || (!root.openRoot(&volume)) )
    LogLungo(LOG_SD_ERROR);
  // Inizializza ESP
  char Temp[50];
  unsigned char i,Size;
  while(!file.open(&root, "conf.txt", O_READ))
    Log(LOG_SD_ERROR);
  // Lettura SSID
  Size = LeggiRiga(file, Temp) - 1;
  char* Ssid = (char*)malloc(Size + 1);
  for(i=0;i<Size;i++)
    Ssid[i] = Temp[i];
  Ssid[Size] = '\0';
  // Lettura PASS
  Size = LeggiRiga(file, Temp) - 1;
  char* Pass = (char*)malloc(Size + 1);
  for(i=0;i<Size;i++)
    Pass[i] = Temp[i];
  Pass[Size] = '\0';
  Size = LeggiRiga(file, Temp) - 1;
  while(!Fishino.begin(Ssid, Pass));
  if(atol(Temp) == 0)
    Fishino.staStartDHCP();
  else
    Fishino.config(atol(Temp));
  while(Fishino.status() != STATION_GOT_IP)
  {
    if( (Fishino.status() == STATION_CONNECT_FAIL) || (Fishino.status() == STATION_TIMEOUT) || (Fishino.status() == STATION_BAD_SSID) )
      Log(LOG_CONNECT_ERROR);
    if(Fishino.status() == STATION_NO_AP_FOUND)
      Log(LOG_NO_AP_FOUND);
    if(Fishino.status() == STATION_WRONG_PASSWORD)
      Log(LOG_PASSWORD_ERROR);
  }
  free(Ssid);
  free(Pass);
  file.close();
  FishinoWebPutHandler::put_handler_fn = file_uploader_handler;
  web.begin();
}

void LoopWeb()
{
  web.process();
}
